'use strict';
const {
  IMPORT_LINE,
  IMPORT_POLYMORPHIC_LINE,
  IMPORT_POLYMORPHIC_INTERFACE_LINE,
  entityLine,
  CLOSE_ENTITY,
  generateClassNameEntity,
  generateColumnLines,
  generateRelationsList,
  generateImportModel,
  generateArrayRelations,
  generatePrimaryColumn,
} = require('./utils/generate-body-entity');
const { checkColumns, colours } = require('./utils/constants');
const { capitalizeFirstLetter } = require('./utils/support');

const generateNestModel = (model, polymorphicRelations) => {

  const { properties, relations, name, base, mixins } = model;

  const arrPropertiesName = Object.keys(properties);
  const arrRelationsName = Object.keys(relations);

  const updatedRelationsName = arrRelationsName.filter(item => arrPropertiesName.includes(item));
  const polymorphicArr = Object.values(relations).filter(item => item.polymorphic);

  const generatedEntity = [
    IMPORT_LINE,
    IMPORT_POLYMORPHIC_LINE,
    IMPORT_POLYMORPHIC_INTERFACE_LINE,
    entityLine(name),
    generateClassNameEntity(name, polymorphicArr),
  ];

  if (!properties) {
    console.log(colours.fg.blue, "no properties detected", colours.reset);
  } else {

    if (Object.keys(properties).includes('id')) {
      const { type } = properties.id;
      generatedEntity.push(generatePrimaryColumn(type));
    } else {
      generatedEntity.push(generatePrimaryColumn());
    }

    Object.keys(properties).map((property, index) => {
      const { postgresql, default: defaultValue } = Object.values(properties)[index];

      let { type } = Object.values(properties)[index];

      if (!type) {
        type = Object.values(properties)[index];
      }

      if (property !== 'id') {
        generatedEntity.push(generateColumnLines(property, type, defaultValue, postgresql));
      }
    });

    // add columns createdAt/updatedAt if not exist
    checkColumns.map(item => {
      if (!Object.keys(properties).includes(item)) {
        generatedEntity.push(generateColumnLines(item, 'date', '$now'));
      }
    });
  }

  if (!relations) {
    console.log(colours.fg.blue, "no relations detected", colours.reset);
  } else {
    const polymorphicRelationItems = [];

    Object.keys(relations).map((relation) => {
      const relationName = updatedRelationsName.includes(relation) ? capitalizeFirstLetter(relation) : relation;
      polymorphicRelations.map(items => {
        items.polymorphic.map(item => {
          return item.polymorphic === relationName ? polymorphicRelationItems.push({ modelName: items.modelName }) : item;
        });
      });
    });

    const arrayRelations = generateArrayRelations(relations, updatedRelationsName);
    const uniqArr = [
      ...new Map(arrayRelations.map(item => [item.modelName, item])).values(),
      ...polymorphicRelationItems.filter(item => item.modelName)
    ];

    // uniq import list
    uniqArr.map(item => {
      const { modelName } = item;
      if (name !== modelName) {
        generatedEntity.splice(1, 0, generateImportModel(modelName));
      }
    });

    const { CascadeDelete: cascadeDeleteArr } = mixins;

    // generating columns
    arrayRelations.map(item => {
      const { modelName, type, relation: relationName, params } = item;
      if (params.polymorphic && typeof params.polymorphic !== 'string') {
        generatedEntity.push(
          generateColumnLines(
            'entityId',
            'number',
            undefined,
            undefined,
            relationName + 'Id',
          ),
          generateColumnLines(
            'entityType',
            'string',
            undefined,
            undefined,
            relationName + 'Type',
          ),
        );
      }

      generatedEntity.push(
        generateRelationsList(
          modelName,
          type,
          name,
          relationName,
          base,
          params,
          cascadeDeleteArr,
          polymorphicRelations,
        ));
    });
  }

  generatedEntity.push(CLOSE_ENTITY);
  return generatedEntity;
};

module.exports = generateNestModel;