#!/usr/bin/env node
'use strict';
const { program } = require('commander');

const {
  customReadFile,
  customMkdirSync,
  customWriteFile,
  customReaddirSync,
} = require('./utils/file-system');

const generateNestModel = require('./generateNestModel');
const { colours } = require('./utils/constants');

program
.version('1.0.0', '-v, --version', 'output the current version')
.requiredOption('--bep, --backend-path <type>', 'Path where the folder with the models is located')
.option('--save, --save-path <type>', 'Path where we will save entity');
program.parse();

const options = program.opts();

// compulsory paths
const { backendPath } = options;
// optional path
const { savePath } = options;

if (backendPath) {
  const modelsPath = backendPath + '/models';
  //'../api/server/model-config.json'
  customReadFile(backendPath + '/model-config.json', (err, data) => {
    if (!err) {
      const arrFolderNames = Object.entries(JSON.parse(data)).filter(item => item[1].dataSource === 'db');
      Promise.all(
        arrFolderNames.map(item => {
          const itemName = item[0];
          return new Promise((resolve, reject) => {
            customReadFile(`${modelsPath}/${itemName}/${itemName}.json`, (err, data) => {
              if (!err) {
                const modelData = JSON.parse(data);
                const { relations } = modelData;
                const polymorphicRelations = Object.values(relations).filter(item => item.polymorphic);
                if (polymorphicRelations.length) {
                  resolve({
                    modelName: itemName,
                    polymorphic: polymorphicRelations,
                  });
                }
                resolve(undefined);
              } else {
                resolve(undefined);
              }
            });
          });
        }),
      ).then((data) => {
        const polymorphicRelations = data.filter(item => item);
        arrFolderNames.map(item => {
          const itemName = item[0];
          //'../api/server/models'
          customReadFile(`${modelsPath}/${itemName}/${itemName}.json`, (err, data) => {
            if (err) {
              console.log(colours.fg.red, `In the folder ${modelsPath} I can't find`, colours.fg.green, itemName, colours.fg.red, 'model');
            } else {
              const destinationFolder = savePath ? `${savePath}/src/${itemName}/entities` : `src/${itemName}/entities`;
              const modelData = JSON.parse(data);
              const { properties, relations } = modelData;
              if (properties && relations) {
                try {
                  customMkdirSync(destinationFolder, { recursive: true });
                  const nestModel = generateNestModel(modelData, polymorphicRelations).join('\n');
                  customWriteFile(`${destinationFolder}/${itemName}.entity.ts`, nestModel, (err) => {
                    if (err) {
                      console.log(err);
                    }
                    console.log(colours.fg.green, itemName, colours.reset, 'was successful formed and written to file');
                  });
                } catch (e) {
                  console.log('Cannot create folder ', e);
                }
              }
            }
          });
        });
      });
    } else {
      console.log(colours.fg.red, 'Make sure that you enter the correct path to the', colours.fg.green, '--conf-path', colours.reset);
    }
  });
}