'use strict';

const { setJoinColumns, generateColumnsName, setCurrentRelation } = require('./relations-support');

const BelongsTo = 'belongsTo';
const HasOne = 'hasOne';
const HasMany = 'hasMany';
const ManyToMany = 'manyToMany';

const mainSettingsForRelation = (
  relationType,
  baseName,
  modelName,
  relationName,
  nameFromModel,
  params,
  cascadeDeleteArr,
  polymorphicRelations,
) => {
  const settingsArray = [];
  const polymorphicRelationItems = [];

  polymorphicRelations.map(items => {
    items.polymorphic.map(item => {
      return item.polymorphic === relationName ? polymorphicRelationItems.push({ name: items.modelName }) : item;
    });
  });

  settingsArray.push(
    setCurrentRelation(
      relationType,
      baseName,
      modelName,
      relationName,
      nameFromModel,
      cascadeDeleteArr,
      params.polymorphic,
      polymorphicRelationItems,
    ),
  );

  if (relationType === 'OneToOne') {
    settingsArray.push(setJoinColumns(params));
  }

  settingsArray.push(
    generateColumnsName(
      modelName,
      relationName,
      baseName,
      nameFromModel,
      relationType,
      params.polymorphic,
      polymorphicRelationItems,
    ), '');
  return settingsArray.join('\n');
};

const selectCurrentRelation = (modelName, type, name, relationName, baseName, params, cascadeDeleteArr, polymorphicRelations) => {
  switch (type) {
    case BelongsTo:
      return mainSettingsForRelation('ManyToOne', baseName, modelName, relationName, name, params, cascadeDeleteArr, polymorphicRelations);
    case HasOne:
      return mainSettingsForRelation('OneToOne', baseName, modelName, relationName, name, params, cascadeDeleteArr, polymorphicRelations);
    case HasMany:
      return mainSettingsForRelation('OneToMany', baseName, modelName, relationName, name, params, cascadeDeleteArr, polymorphicRelations);
    case ManyToMany:
      return mainSettingsForRelation('ManyToMany', baseName, modelName, relationName, name, params, cascadeDeleteArr, polymorphicRelations);
    default:
      return `please add logic for ${type}`;
  }
};

module.exports = {
  selectCurrentRelation,
};