'use strict';

const { capitalizeFirstLetter } = require('../utils/support');
const { childLists, customFields, additionalSpaces, colours } = require('../utils/constants');

const setCurrentRelation = (
  relationType,
  baseName,
  modelName,
  relationName,
  nameFromModel,
  cascadeDeleteArr,
  polymorphicItem,
  polymorphicRelations,
) => {
  const useName = modelName || nameFromModel;
  const capitalizeName = capitalizeFirstLetter(useName);

  let fullRelationName = useName + '.' + (customFields[nameFromModel] || nameFromModel);

  if (polymorphicItem) {
    if (typeof polymorphicItem !== 'string') {
      if (polymorphicRelations.length) {
        return additionalSpaces + `@PolymorphicParent(() => [${polymorphicRelations.map(item => capitalizeFirstLetter(item.name))}])`;
      } else {
        console.log(colours.fg.red, "Sorry, no polymorphic relations", colours.reset);
        return '';
      }
    } else {
      return additionalSpaces + `@PolymorphicChildren(() => ${capitalizeName})`;
    }
  }

  if (relationName === 'parent') {
    fullRelationName = nameFromModel + '.parent' + capitalizeName;
    return additionalSpaces + `@${relationType}(() => ${capitalizeName}, (${useName}: any) => ${fullRelationName})`;
  }

  if (childLists.includes(relationName)) {
    fullRelationName = nameFromModel + '.child' + capitalizeName;
    return additionalSpaces + `@${relationType}(() => ${capitalizeName}, (${useName}: any) => ${fullRelationName})`;
  }

  return additionalSpaces + `@${relationType}(() => ${capitalizeName}, (${useName}: any) => ${fullRelationName}${
    cascadeDeleteArr && cascadeDeleteArr.relations.includes(useName) ? ', { cascade: ["update", "remove"] }' : ''
  })`;
};

const generateJoinColumn = (params) => {
  const { foreignKey } = params;

  if (foreignKey) {
    return `{ name: '${foreignKey}' }`;
  }
  return '';
};

const setJoinColumns = (params) => {
  return additionalSpaces + `@JoinColumn(${generateJoinColumn(params)})`;
};

const generateColumnsName = (
  modelName,
  relationName,
  baseName,
  nameFromModel,
  relationType,
  polymorphicItem,
  polymorphicRelations,
) => {
  const useName = modelName || nameFromModel;
  const capitalizeName =
    polymorphicItem && typeof polymorphicItem !== 'string'
      ? polymorphicRelations.map(item => capitalizeFirstLetter(item.name)).toString().replace(/,/, ' | ')
      : capitalizeFirstLetter(useName);

  if (!capitalizeName) {
    return '';
  }

  if (relationName === 'parent') {
    return additionalSpaces + `parent${capitalizeName}: ${capitalizeName};`;
  }

  if (childLists.includes(relationName)) {
    return additionalSpaces + `child${capitalizeName}: ${capitalizeName}[];`;
  }

  if (relationType === 'ManyToMany' || relationType === 'OneToMany') {
    return additionalSpaces + `${relationName}: ${capitalizeName}[];`;
  }

  return additionalSpaces + `${relationName}: ${capitalizeName};`;
};

module.exports = {
  setJoinColumns,
  generateColumnsName,
  setCurrentRelation,
};