'use strict';
const { types } = require('./constants');

const capitalizeFirstLetter = (string) => {
  return string.charAt(0).toUpperCase() + string.slice(1);
};

const setDefaultValue = (item, type) => {
  if (item !== undefined && type === 'string') {
    return `'${item}'`;
  }

  if (type === 'date') {
    return `() => 'NOW()'`;
  }

  if (Array.isArray(item) && item.length === 0) {
    return '[]';
  }
  return item;
};

const setCurrentColumnType = (type, postgresql) => {
  if (Array.isArray(type) && type.length) {
    return `${type[0] === 'object' ? '{}' : type[0]}[]`;
  }

  if (postgresql) {
    const { dataType } = postgresql;
    if (types.some(element => element === dataType)) {
      return '{}';
    }
  }
  if (type === 'date') {
    return 'Date'
  }

  if (type === 'object') {
    return '{}';
  }

  if (type === 'array') {
    return '[]'
  }
  return type;
};

module.exports = {
  capitalizeFirstLetter,
  setDefaultValue,
  setCurrentColumnType,
};